@@ -1,4 +1,4 @@
-/* $Id: main.c 29153 2009-01-11 14:41:07Z nick $ */
+/* $Id$ */
 /*
  *  Copyright (c) 2008 Stephan Arts <stephan@xfce.org>
  *  Copyright (c) 2008 Jannis Pohlmann <jannis@xfce.org>
@@ -672,8 +672,6 @@ appearance_settings_dialog_configure_widgets (GladeXML *gxml)
     GtkListStore     *list_store;
     GtkCellRenderer  *renderer;
     GtkTreeSelection *icon_selection, *ui_selection;
-    gchar            *string;
-    guint             i;
     GdkPixbuf        *pixbuf;
 
     /* check if we need to restore settings from GtkSettings */
