@@ -32,13 +32,14 @@
 
 #define WORKSPACES_CHANNEL    "xfwm4"
 #define WORKSPACE_NAMES_PROP  "/general/workspace_names"
-
+#define WORKSPACE_COUNT_PROP  "/general/workspace_count"
 
 struct _XfceWorkspacesHelper
 {
     GObject parent;
 
     WnckScreen *screen;
+
     XfconfChannel *channel;
 };
 
@@ -50,7 +51,7 @@ typedef struct _XfceWorkspacesHelperClass
 static void xfce_workspaces_helper_class_init(XfceWorkspacesHelperClass *klass);
 static void xfce_workspaces_helper_init(XfceWorkspacesHelper *helper);
 static void xfce_workspaces_helper_finalize(GObject *obj);
-
+static void xfce_workspaces_helper_set_workspace_names(XfceWorkspacesHelper *helper);
 static void xfce_workspaces_helper_update_all_names(XfceWorkspacesHelper *helper);
 static void xfce_workspaces_helper_prop_changed(XfconfChannel *channel,
                                                 const gchar *property,
@@ -72,17 +73,11 @@ xfce_workspaces_helper_class_init(XfceWorkspacesHelperClass *klass)
 static void
 xfce_workspaces_helper_init(XfceWorkspacesHelper *helper)
 {
-    WnckWorkspace *space;
-    gint i, n_workspaces;
-    gchar **names;
-
     helper->channel = xfconf_channel_new(WORKSPACES_CHANNEL);
-    names = xfconf_channel_get_string_list(helper->channel,
-                                           WORKSPACE_NAMES_PROP);
 
-    /* FIXME: need to do this for all screens? */
+    xfce_workspaces_helper_set_workspace_names(helper);
+
     helper->screen = wnck_screen_get_default();
-    wnck_screen_force_update(helper->screen);
     g_signal_connect_swapped(G_OBJECT(helper->screen), "workspace-created",
                              G_CALLBACK(xfce_workspaces_helper_update_all_names),
                              helper);
@@ -90,32 +85,9 @@ xfce_workspaces_helper_init(XfceWorkspacesHelper *helper)
                              G_CALLBACK(xfce_workspaces_helper_update_all_names),
                              helper);
 
-    n_workspaces = wnck_screen_get_workspace_count(helper->screen);
-    i = 0;
-    if(names) {
-        for(; i < n_workspaces && names[i]; ++i) {
-            space = wnck_screen_get_workspace(helper->screen, i);
-            wnck_workspace_change_name(space, names[i]);
-        }
-    }
-    if(i != n_workspaces) {
-        /* some of them may not have been set in xfconf */
-        names = g_realloc(names, sizeof(gchar *) * (n_workspaces + 1));
-        for(; i < n_workspaces; ++i) {
-            names[i] = g_strdup_printf(_("Workspace %d"), i + 1);
-            space = wnck_screen_get_workspace(helper->screen, i);
-            wnck_workspace_change_name(space, names[i]);
-        }
-        names[i] = NULL;
-        xfconf_channel_set_string_list(helper->channel, WORKSPACE_NAMES_PROP,
-                                       (const gchar **)names);
-    }
-
     g_signal_connect(G_OBJECT(helper->channel),
                      "property-changed::" WORKSPACE_NAMES_PROP,
                      G_CALLBACK(xfce_workspaces_helper_prop_changed), helper);
-
-    g_strfreev(names);
 }
 
 static void
@@ -127,32 +99,83 @@ xfce_workspaces_helper_finalize(GObject *obj)
                                          G_CALLBACK(xfce_workspaces_helper_update_all_names),
                                          helper);
 
-    g_signal_handlers_disconnect_by_func(G_OBJECT(helper->channel),
-                                         G_CALLBACK(xfce_workspaces_helper_prop_changed),
-                                         helper);
     g_object_unref(G_OBJECT(helper->channel));
 
     G_OBJECT_CLASS(xfce_workspaces_helper_parent_class)->finalize(obj);
 }
 
+static void
+xfce_workspaces_helper_set_workspace_names(XfceWorkspacesHelper *helper)
+{
+    WnckWorkspace *workspace;
+    WnckScreen *screen;
+    GList *li, *workspaces;
+    guint i, n_names;
+    gchar **names;
+    gint n_screens, n;
+    const gchar *name, *old_name;
+    gchar *tmp_name;
+    
+    /* get the workspace names */
+    names = xfconf_channel_get_string_list(helper->channel, WORKSPACE_NAMES_PROP);
+    n_names = names ? g_strv_length(names) : 0;
 
+    /* walk all the displays on this screen */
+    /* FIXME? Is this really needed? */
+    n_screens = gdk_display_get_n_screens(gdk_display_get_default());
+    for(n = 0; n < n_screens; n++) {
+        /* get the wnck screen and force an update */
+        screen = wnck_screen_get(n);
+        wnck_screen_force_update(screen);
+
+        /* walk all the workspaces on this screen */
+        workspaces = wnck_screen_get_workspaces(screen);
+        for(li = workspaces, i = 0; li != NULL; li = li->next, i++) {
+            workspace = WNCK_WORKSPACE(li->data);
+
+            /* check if we have a valid name in the array */
+            if(n_names > i && names[i] != NULL && names[i] != '\0') {
+                name = names[i];
+                tmp_name = NULL;
+            } else {
+                tmp_name = g_strdup_printf(_("Workspace %d"), i + 1);
+                name = tmp_name;
+            }
+
+            /* update the workspace name if it has changed */
+            old_name = wnck_workspace_get_name(workspace);
+            if(old_name == NULL || strcmp(old_name, name) != 0)
+                wnck_workspace_change_name(workspace, name);
+
+            /* cleanup */
+            g_free(tmp_name);
+        }
+    }
+    
+    /* cleanup */
+    g_strfreev(names);
+}
 
 static void
 xfce_workspaces_helper_update_all_names(XfceWorkspacesHelper *helper)
 {
-    gint i, n_workspaces = wnck_screen_get_workspace_count(helper->screen);
+    gint i, n_workspaces;
     gchar const **names;
+    WnckWorkspace *workspace;
 
-    names = g_malloc(sizeof(gchar *) * (n_workspaces + 1));
+    g_return_if_fail (XFCE_IS_WORKSPACES_HELPER (helper));
+
+    n_workspaces = wnck_screen_get_workspace_count(helper->screen);
+    names = g_new0(gchar const *, n_workspaces + 1);
 
     for(i = 0; i < n_workspaces; ++i) {
-        WnckWorkspace *space = wnck_screen_get_workspace(helper->screen, i);
-        names[i] = wnck_workspace_get_name(space);
+        workspace = wnck_screen_get_workspace(helper->screen, i);
+        names[i] = wnck_workspace_get_name(workspace);
     }
-    names[n_workspaces] = NULL;
 
     xfconf_channel_set_string_list(helper->channel, WORKSPACE_NAMES_PROP,
                                    (const gchar **)names);
+    xfconf_channel_set_int(helper->channel, WORKSPACE_COUNT_PROP, n_workspaces);
 
     g_free(names);
 }
@@ -163,37 +186,6 @@ xfce_workspaces_helper_prop_changed(XfconfChannel *channel,
                                     const GValue *value,
                                     gpointer user_data)
 {
-    XfceWorkspacesHelper *helper = user_data;
-    GPtrArray *names;
-    guint i, n_workspaces = wnck_screen_get_workspace_count(helper->screen);
-
-    if(G_VALUE_TYPE(value) !=  dbus_g_type_get_collection("GPtrArray",
-                                                          G_TYPE_VALUE))
-    {
-        g_warning("(workspace names) Expected boxed GPtrArray property, got %s",
-                  G_VALUE_TYPE_NAME(value));
-        return;
-    }
-
-    names = g_value_get_boxed(value);
-    if(!names)
-        return;
-
-    for(i = 0; i < n_workspaces && i < names->len; ++i) {
-        WnckWorkspace *space = wnck_screen_get_workspace(helper->screen, i);
-        GValue *val = g_ptr_array_index(names, i);
-        const gchar *old_name, *new_name;
-
-        if(!G_VALUE_HOLDS_STRING(val)) {
-            g_warning("(workspace names) Expected string but got %s for item %d",
-                      G_VALUE_TYPE_NAME(val), i);
-            continue;
-        }
-
-        /* only update the names that have actually changed */
-        old_name = wnck_workspace_get_name(space);
-        new_name = g_value_get_string(val);
-        if(strcmp(old_name, new_name))
-            wnck_workspace_change_name(space, new_name);
-    }
+    g_return_if_fail (XFCE_IS_WORKSPACES_HELPER (user_data));
+    xfce_workspaces_helper_set_workspace_names(XFCE_WORKSPACES_HELPER(user_data));
 }
